/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include "mainframe.hpp"

#include "gtkmm/separatormenuitem.h"
#include "gtkmm/table.h"
#include "gtkmm/frame.h"
#include "gtkmm/alignment.h"
#include "gtkmm/messagedialog.h"
#include "gtkmm/stockid.h"
#include "gtkmm/stock.h"
#include "gtkmm/radiobuttongroup.h"

#include <sstream>
#include <iostream>

using namespace Gtk;

namespace Chess{

	MainFrame::MainFrame()
	:str_player_blue_pref(Glib::locale_to_utf8("/")),
		str_player_red_pref(Glib::locale_to_utf8("/")),
		str_player1_pref(Glib::locale_to_utf8("1/")),
		str_player2_pref(Glib::locale_to_utf8("2/")){

		loadImages();

		VBox* box1 = manage(new VBox());
		menubar = manage(new MenuBar());
		menu_game = manage(new Menu());
		menu_about = manage(new Menu());
		Table* table1 = manage(new Table(3, 2));
		Frame *frame = manage(new Frame(Glib::locale_to_utf8("a")));
		Alignment *ali1 = manage(new Alignment(Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER, 0.0f, 0.0f));
		board = manage(new Board(*this));
		Label* lbl_last_rm = manage(new Label(Glib::locale_to_utf8("W:")));
		img_last_rm = manage(new Image("images/none.gif"));
		HBox *box2 = manage(new HBox(false, 12));
		img_player = manage(new Image(pxb_player_unknown));
		lbl_player_blue_title = manage(new Label(str_player_blue_pref));
		lbl_player1_title = lbl_player_blue_title;
		lbl_player_blue = manage(new Label("0"));
		lbl_player1 = lbl_player_blue;
		lbl_player_red_title = manage(new Label(str_player_red_pref));
		lbl_player2_title = lbl_player_red_title;
		lbl_player_red = manage(new Label("0"));
		lbl_player2 = lbl_player_red;
		status = manage(new Statusbar());
		box1->pack_start(*menubar, PACK_SHRINK);
		box1->pack_start(*table1, PACK_EXPAND_WIDGET);
		box1->pack_start(*status, PACK_SHRINK);
		menubar->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8("C [_G]"), *menu_game));
		menubar->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8(" [_A]"), *menu_about));
		menu_game->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8("sC [_N]"), Gtk::AccelKey("<control>n"), sigc::mem_fun(*this, &MainFrame::onNew)));
		menu_game->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8("o [_E]"), sigc::mem_fun(*this, &MainFrame::onDrop)));
		menu_game->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
		menu_game->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8("C [_X]"), Gtk::AccelKey("<control>x"), sigc::mem_fun(*this, &MainFrame::onExit)));
		menu_about->items().push_back(Gtk::Menu_Helpers::MenuElem(Glib::locale_to_utf8("st [_A]"), Gtk::AccelKey("F2"), sigc::mem_fun(*this, &MainFrame::onAbout)));
		table1->attach(*frame, 0, 2, 0, 1, FILL|EXPAND, FILL);
		table1->attach(*ali1, 0, 1, 1, 3);
		table1->attach(*lbl_last_rm, 1, 2, 1, 2, FILL, EXPAND, 12);
		table1->attach(*img_last_rm, 1, 2, 2, 3, FILL, EXPAND);
		frame->add(*box2);
		box2->pack_start(*img_player, PACK_SHRINK, 5);
		box2->pack_end(*lbl_player_red, PACK_SHRINK, 12);
		box2->pack_end(*lbl_player_red_title, PACK_SHRINK);
		box2->pack_end(*lbl_player_blue, PACK_SHRINK, 12);
		box2->pack_end(*lbl_player_blue_title, PACK_SHRINK);
		ali1->add(*board);
		add(*box1);
		show_all_children();
		set_title(Glib::locale_to_utf8("st"));

		player_blue_remains = player_red_remains = player1_remains = player2_remains = 0;
		turn = UNKNOWN;

		local_rule = 0;
	}

	void MainFrame::loadImages(){
		pxb_player_unknown = Gdk::Pixbuf::create_from_file("images/player_unknown.gif");
		pxb_player_blue = Gdk::Pixbuf::create_from_file("images/player_blue.gif");
		pxb_player_red = Gdk::Pixbuf::create_from_file("images/player_red.gif");
		pxb_player1 = Gdk::Pixbuf::create_from_file("images/player1.gif");
		pxb_player2 = Gdk::Pixbuf::create_from_file("images/player2.gif");
	}

	void MainFrame::onNew(){
		NewGameDialog ngd;
		if(ngd.run() == NewGameDialog::ID_OK){
			NewGameDialog::RuleSelection sr = ngd.getSelectedRule();
			if(sr == NewGameDialog::NONE)
				return;
			else if(sr == NewGameDialog::BASERULE){
				delete local_rule;
				local_rule = new BaseRule();
			}
			else{
				delete local_rule;
				local_rule = new ExtRule();
			}
			Glib::ustring name1(ngd.getName1()), name2(ngd.getName2());
			Player p1(name1);
			Player p2(name2);
			local_rule->addOutputListener(this);
			local_rule->start(p1, p2);
			setInputListener(local_rule);
		}
	}

	void MainFrame::onDrop(){
		if(local_rule)
			local_rule->dropGame();
	}

	void MainFrame::onExit(){
		hide();
	}

	void MainFrame::onAbout(){
		MessageDialog dialog(*this, Glib::locale_to_utf8("st 1.0  Copyright(c) 2005\n\n@: ^aO\n\n}XtѹCAGNU GPL v2.0X\nXG2005/8/22\n\nkγWhԨREADME"), false, MESSAGE_INFO, BUTTONS_CLOSE, true);
		dialog.set_title(Glib::locale_to_utf8(""));
		dialog.run();
	}

	void MainFrame::start(int n_cols, int n_rows, std::vector<Chess> chesses){
		turn = UNKNOWN;
		board->start(n_cols, n_rows, chesses);
		status->pop();
		status->push("");
	}

	void MainFrame::end(EndStatus status_){
		status->pop();
		switch(status_){
			case PLAYER_BLUE_WIN:
				status->push(lbl_player_blue_title->get_text() + Glib::locale_to_utf8(" ĹF!"));
				img_player->set(pxb_player_unknown);
				break;
			case PLAYER_RED_WIN:
				status->push(lbl_player_red_title->get_text() + Glib::locale_to_utf8(" ĹF!"));
				img_player->set(pxb_player_unknown);
				break;
			case PLAYER_1_WIN:
				status->push(lbl_player1_title->get_text() + Glib::locale_to_utf8(" ĹF!"));
				img_player->set(pxb_player_unknown);
				break;
			case PLAYER_2_WIN:
				status->push(lbl_player2_title->get_text() + Glib::locale_to_utf8(" ĹF!"));
				img_player->set(pxb_player_unknown);
				break;
			case DROP:
				status->push(Glib::locale_to_utf8("ѧ"));
				img_player->set(pxb_player_unknown);
				break;
		}
		board->end(status_);
	}

	void MainFrame::setPlayerName(Turn t, Glib::ustring name){
		std::ostringstream output;
		switch(t){
			case PLAYER_1:
				lbl_player1_title->set_text(str_player1_pref + name + ":");
				output << player1_remains;
				lbl_player1->set_text(output.str());
				break;
			case PLAYER_2:
				lbl_player2_title->set_text(str_player2_pref + name + ":");
				output << player2_remains;
				lbl_player2->set_text(output.str());
				break;
			case PLAYER_BLUE:
				lbl_player_blue_title->set_text(str_player_blue_pref + name + ":");
				output << player_blue_remains;
				lbl_player_blue->set_text(output.str());
				break;
			case PLAYER_RED:
				lbl_player_red_title->set_text(str_player_red_pref + name + ":");
				output << player_red_remains;
				lbl_player_red->set_text(output.str());
				break;
		}
	}

	void MainFrame::setCurrentPlayer(Turn t){
		switch(t){
			case PLAYER_1:
				img_player->set(pxb_player1);
				break;
			case PLAYER_2:
				img_player->set(pxb_player2);
				break;
			case PLAYER_BLUE:
				img_player->set(pxb_player_blue);
				break;
			case PLAYER_RED:
				img_player->set(pxb_player_red);
				break;
		}
	}

	void MainFrame::updatePlayer(Turn t, int remains){
		std::ostringstream output;
		output << remains;
		switch(t){
			case PLAYER_1:
				player1_remains = remains;
				lbl_player1->set_text(output.str());
				break;
			case PLAYER_2:
				player2_remains = remains;
				lbl_player2->set_text(output.str());
				break;
			case PLAYER_BLUE:
				player_blue_remains = remains;
				lbl_player_blue->set_text(output.str());
				break;
			case PLAYER_RED:
				player_red_remains = remains;
				lbl_player_red->set_text(output.str());
				break;
		}
	}

	void MainFrame::moveChess(int x1, int y1, int x2, int y2){
		board->moveChess(x1, y1, x2, y2);
	}

	void MainFrame::uncoverChess(int x, int y){
		board->uncoverChess(x, y);
	}

	void MainFrame::removeChess(int x, int y){
		board->removeChess(x, y);
	}

	void MainFrame::setSelectedChess(int x, int y, bool select){
		board->setSelectedChess(x, y, select);
	}

	void MainFrame::setMessage(Glib::ustring message){
		status->pop();
		status->push(message);
	}

	void MainFrame::setLastRemovedImage(Glib::RefPtr<Gdk::Pixbuf> img){
		img_last_rm->set(img);
	}

	void MainFrame::setInputListener(InputListener* il){
		board->setInputListener(il);
	}

	Board::Board(MainFrame& p): parent(p){
		set_size_request(481, 241);
		loadImages();
		selected = 0;
		il = 0;
	}

	void Board::loadImages(){
		//Chess::CHESS_XXX Lkѹ骺Ѧ,ҥHoǵ{X|XD
		/*imgs_blue[Chess::CHESS_CHANG]	= Gdk::Pixbuf::create_from_file("images/b_chang.gif");
		imgs_blue[Chess::CHESS_SHI]		= Gdk::Pixbuf::create_from_file("images/b_shi.gif");
		imgs_blue[Chess::CHESS_SHANG]	= Gdk::Pixbuf::create_from_file("images/b_shang.gif");
		imgs_blue[Chess::CHESS_CHU]		= Gdk::Pixbuf::create_from_file("images/b_chu.gif");
		imgs_blue[Chess::CHESS_MA]		= Gdk::Pixbuf::create_from_file("images/b_ma.gif");
		imgs_blue[Chess::CHESS_PAOU]	= Gdk::Pixbuf::create_from_file("images/b_paou.gif");
		imgs_blue[Chess::CHESS_TSU]		= Gdk::Pixbuf::create_from_file("images/b_tsu.gif");

		imgs_red[Chess::CHESS_CHANG]	= Gdk::Pixbuf::create_from_file("images/r_chang.gif");
		imgs_red[Chess::CHESS_SHI]		= Gdk::Pixbuf::create_from_file("images/r_shi.gif");
		imgs_red[Chess::CHESS_SHANG]	= Gdk::Pixbuf::create_from_file("images/r_shang.gif");
		imgs_red[Chess::CHESS_CHU]		= Gdk::Pixbuf::create_from_file("images/r_chu.gif");
		imgs_red[Chess::CHESS_MA]		= Gdk::Pixbuf::create_from_file("images/r_ma.gif");
		imgs_red[Chess::CHESS_PAOU]		= Gdk::Pixbuf::create_from_file("images/r_paou.gif");
		imgs_red[Chess::CHESS_TSU]		= Gdk::Pixbuf::create_from_file("images/r_tsu.gif");*/
		loadChessImage(imgs_blue, Chess::CHESS_CHANG,	"images/b_chang.gif");
		loadChessImage(imgs_blue, Chess::CHESS_SHI,		"images/b_shi.gif");
		loadChessImage(imgs_blue, Chess::CHESS_SHANG,	"images/b_shang.gif");
		loadChessImage(imgs_blue, Chess::CHESS_CHU,		"images/b_chu.gif");
		loadChessImage(imgs_blue, Chess::CHESS_MA,		"images/b_ma.gif");
		loadChessImage(imgs_blue, Chess::CHESS_PAOU,	"images/b_paou.gif");
		loadChessImage(imgs_blue, Chess::CHESS_TSU,		"images/b_tsu.gif");

		loadChessImage(imgs_red, Chess::CHESS_CHANG,	"images/r_shuai.gif");
		loadChessImage(imgs_red, Chess::CHESS_SHI,		"images/r_shi.gif");
		loadChessImage(imgs_red, Chess::CHESS_SHANG,	"images/r_shang.gif");
		loadChessImage(imgs_red, Chess::CHESS_CHU,		"images/r_chu.gif");
		loadChessImage(imgs_red, Chess::CHESS_MA, 		"images/r_ma.gif");
		loadChessImage(imgs_red, Chess::CHESS_PAOU,		"images/r_paou.gif");
		loadChessImage(imgs_red, Chess::CHESS_TSU,		"images/r_bin.gif");

		img_uncovered = Gdk::Pixbuf::create_from_file("images/uncovered.gif");
		img_none = Gdk::Pixbuf::create_from_file("images/none.gif");
	}

	void Board::loadChessImage(std::map<int, Glib::RefPtr<Gdk::Pixbuf> >& imgs_map, int id, std::string img_file){
		imgs_map[id] = Gdk::Pixbuf::create_from_file(img_file);
	}

	void Board::on_realize(){
		Gtk::DrawingArea::on_realize();

		color_bg.set_rgb(255*256, 255*256, 200*256);
		color_fg.set_rgb(168*256, 168*256, 168*256);
		color_select.set_rgb(255*256, 215*256, 0);
		Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap();
		colormap->alloc_color(color_bg);
		colormap->alloc_color(color_fg);
		colormap->alloc_color(color_select);

		window = get_window();
		window->set_background(color_bg);
		gc = Gdk::GC::create(window);
		/*select_gc = Gdk::GC::create(window);
		select_gc->set_line_attributes(1, Gdk::LINE_ON_OFF_DASH, Gdk::CAP_NOT_LAST, Gdk::JOIN_BEVEL);
		select_gc->set_foreground(color_fg);*/

		add_events(Gdk::BUTTON_PRESS_MASK);
	}

	bool Board::on_expose_event(GdkEventExpose* e){
		window->clear();
		gc->set_foreground(color_fg);
		//ުu
		for(int i = 0; i <= 8; ++i)
			window->draw_line(gc, i*60, 0, i*60, 240);
		//޾u
		for(int i = 0; i <= 4; ++i)
			window->draw_line(gc, 0, i*60, 480, i*60);
		//eѤl
		for(int col = 0; col < 8; ++col)
			for(int row = 0; row < 4; ++row)
				drawChess(col, row);
		return true;
	}

	bool Board::on_button_press_event(GdkEventButton* e){
		//std::cout << "button " << e->button << " was pressed at (" << e->x << "," << e->y << ")" << std::endl;
		int col = static_cast<int>(e->x) / 60;
		int row = static_cast<int>(e->y) / 60;
		///
		//std::cout << "col=" << col << "  row=" << row << std::endl;
		///
		if(il && col >= 0 && col < 8 && row >= 0 && row < 8){
			if(e->button == 1)
				il->clicked(col, row, InputListener::LEFT_BUTTON);
			else if(e->button == 3)
				il->clicked(col, row, InputListener::RIGHT_BUTTON);
		}
		return true;
	}

	void Board::drawChess(int col, int row){
		Chess* chess = chesses[col][row];

		if(!chess)
			return;

		int id = chess->getID();
		switch(id){
			case Chess::CHESS_CHANG:
			case Chess::CHESS_SHI:
			case Chess::CHESS_SHANG:
			case Chess::CHESS_CHU:
			case Chess::CHESS_MA:
			case Chess::CHESS_PAOU:
			case Chess::CHESS_TSU:
				if(chess == selected){
					gc->set_foreground(color_select);
					window->draw_rectangle(gc, true, col*60+1, row*60+1, 58, 58);
				}
				if(!chess->isCovered()){
					img_uncovered->render_to_drawable(
							window, get_style()->get_black_gc(),
							0, 0,
							col*60+6, row*60+6,
							img_uncovered->get_width(), img_uncovered->get_height(),
							Gdk::RGB_DITHER_NONE, 0, 0
					);
				}
				else if(chess->getColor() == Chess::BLUE){
					Glib::RefPtr<Gdk::Pixbuf> img = imgs_blue[id];
					img->render_to_drawable(
							window, get_style()->get_black_gc(),
							0, 0,
							col*60+6, row*60+6,
							img->get_width(), img->get_height(),
							Gdk::RGB_DITHER_NONE, 0, 0
					);
				}
				else{
					Glib::RefPtr<Gdk::Pixbuf> img = imgs_red[id];
					img->render_to_drawable(
							window, get_style()->get_black_gc(),
							0, 0,
							col*60+6, row*60+6,
							img->get_width(), img->get_height(),
							Gdk::RGB_DITHER_NONE, 0, 0
					);
				}
				break;
		}
	}

	void Board::redraw(){
		queue_draw();
	}

	void Board::redraw(int col, int row){
		queue_draw_area(col*60, row*60, 62, 62);
	}

	void Board::setInputListener(InputListener* il_){
		il = il_;
	}

	void Board::start(int n_cols, int n_rows, std::vector<Chess>& chesses_){
		if(n_cols != 8 || n_rows != 4)
			return;
		chesses_vec.clear();
		chesses_vec = chesses_;
		std::vector<Chess>::iterator ite = chesses_vec.begin();
		for(int row = 0; row < 4; ++row)
			for(int col = 0; col < 8; ++col){
				chesses[col][row] = &(*ite);
				ite++;
			}
		selected = 0;
		redraw();
		parent.setLastRemovedImage(img_none);
	}

	void Board::end(OutputListener::EndStatus status){
		/*chesses_vec.clear();
		for(int col = 0; col < 8; ++col)
			for(int row = 0; row < 4; ++row)
				chesses[col][row] = 0;*/
	}

	void Board::setCurrentPlayer(Turn turn){
		selected = 0;
		redraw();
	}

	void Board::moveChess(int x1, int y1, int x2, int y2){
		chesses[x2][y2] = chesses[x1][y1];
		chesses[x1][y1] = 0;
		redraw(x2, y2);
		redraw(x1, y1);
	}

	void Board::uncoverChess(int x, int y){
		chesses[x][y]->cover();
		redraw(x, y);
	}

	void Board::removeChess(int x, int y){
		if(chesses[x][y]){
			if(chesses[x][y]->getColor() == Chess::BLUE)
				parent.setLastRemovedImage(imgs_blue[chesses[x][y]->getID()]);
			else
				parent.setLastRemovedImage(imgs_red[chesses[x][y]->getID()]);
		}
		if(selected == chesses[x][y])
			selected = 0;
		chesses[x][y] = 0;
		redraw(x, y);
	}

	void Board::setSelectedChess(int x, int y, bool select){
		//]̦hu@ӴѤl
		if(select)
			selected = chesses[x][y];
		else
			selected = 0;
		redraw();
	}

	NewGameDialog::NewGameDialog()
	:Dialog(Glib::locale_to_utf8("sC"), true, true){
		StockID sid_cancel(Gtk::Stock::CANCEL), sid_ok(Gtk::Stock::OK);
		add_button(sid_cancel, ID_CANCEL);
		add_button(sid_ok, ID_OK);

		Table* table = manage(new Table(5, 2));
		Label* lbl1 = manage(new Label(Glib::locale_to_utf8("Wh:"), ALIGN_LEFT));
		RadioButtonGroup sel_gp;
		sel_baserule = manage(new RadioButton(sel_gp, Glib::locale_to_utf8("򥻳Wh")));
		sel_extrule = manage(new RadioButton(sel_gp, Glib::locale_to_utf8("Wh")));
		Label* lbl2 = manage(new Label(Glib::locale_to_utf8("aT:"), ALIGN_LEFT));
		Label* lbl3 = manage(new Label(Glib::locale_to_utf8("a@:")));
		txt_player1 = manage(new Entry());
		txt_player1->set_text("Player1");
		Label* lbl4 = manage(new Label(Glib::locale_to_utf8("aG:")));
		txt_player2 = manage(new Entry());
		txt_player2->set_text("Player2");

		table->attach(*lbl1, 0, 2, 0, 1, FILL|EXPAND, FILL, 0, 8);
		table->attach(*sel_baserule, 0, 1, 1, 2, FILL, FILL, 2, 2);
		table->attach(*sel_extrule, 1, 2, 1, 2, FILL, FILL, 2, 2);
		table->attach(*lbl2, 0, 2, 2, 3, FILL|EXPAND, FILL, 0, 8);
		table->attach(*lbl3, 0, 1, 3, 4, FILL, FILL, 2, 2);
		table->attach(*txt_player1, 1, 2, 3, 4, FILL|EXPAND, FILL, 2, 2);
		table->attach(*lbl4, 0, 1, 4, 5, FILL, FILL, 2, 2);
		table->attach(*txt_player2, 1, 2, 4, 5, FILL|EXPAND, FILL, 2, 2);

		get_vbox()->pack_start(*table, PACK_SHRINK, 2);

		show_all_children();
	}

	NewGameDialog::RuleSelection NewGameDialog::getSelectedRule(){
		if(sel_baserule->get_active())
			return BASERULE;
		else if(sel_extrule->get_active())
			return EXTRULE;
		return NONE;
	}

	Glib::ustring NewGameDialog::getName1(){
		return txt_player1->get_text();
	}

	Glib::ustring NewGameDialog::getName2(){
		return txt_player2->get_text();
	}

}

#include "gtkmm/main.h"

int main(int argc, char** argvs){
	Gtk::Main kit(argc, argvs);
	Chess::MainFrame frame;
	kit.run(frame);
	return 0;
}
