/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef RULE_HPP
#define RULE_HPP

#include "chessboard.hpp"
#include "inputlistener.hpp"
#include "outputlistener.hpp"
#include "chess.hpp"
#include "enums.hpp"
#include <list>

namespace Chess{

	class Player;


	//Engine, core
	/**
	 * Class Rule
	 * CDn֤O,tdDɹCi(Wh)αJ/X
	 * Whһ~ӦۦO
	 */
	class Rule: public InputListener{
		protected:
			Rule();
		public:
			/* C}l */
			virtual void start(Player& p1, Player& p2);
			/* C */
			virtual void dropGame();
			virtual ~Rule();
		public:
			enum Status{
				UNINITIALIZED,	/* C|}l */
				UNSTARTED,	/* C}l */
				PLAYING,	/* Ci椤 */
				ENDED	/* C */
			};

		public:
			inline const Player* getPlayer1() { return player1; }
			inline const Player* getPlayer2() { return player2; }
			inline const Player* getPlayerBlue() { return player_blue; }
			inline const Player* getPlayerRed() { return player_red; }
			const Player* getCurrentPlayer();
			Turn getTurn();
			void addOutputListener(OutputListener* ol);
			void removeOutputListener(OutputListener* ol);
			void clearOutputListeners();
			inline Status getStatus() { return status; }

		protected:
			/* HUPUOutputListenerqU禡,w]tX\ */
			void outputStart(int n_cols, int n_rows, std::vector<Chess>& chesses);
			void outputEnd(OutputListener::EndStatus status);
			void outputSetPlayerName(Turn turn, Glib::ustring name);
			void outputUpdatePlayer(Turn turn, int remains);
			void outputSetCurrentPlayer(Turn turn);
			void outputMoveChess(int x1, int y1, int x2, int y2);
			void outputUncoverChess(int x, int y);
			void outputRemoveChess(int x, int y);
			void outputSetSelectedChess(int x, int y, bool select);
			void outputSetMessage(Glib::ustring message);
		protected:
			template<class Arg1Type, class Operation>
			class OutputBinder1{
				public:
					OutputBinder1(Arg1Type _val1)
						:val1(_val1) { }
				protected:
					Arg1Type val1;
					Operation op;
				public:
					void operator()(OutputListener* ol){
						op(ol, val1);
					}
			};

			template<class Arg1Type, class Arg2Type, class Operation>
			class OutputBinder2{
				public:
					OutputBinder2(Arg1Type _val1, Arg2Type _val2)
						:val1(_val1), val2(_val2) { }
				protected:
					Arg1Type val1;
					Arg2Type val2;
					Operation op;
				public:
					void operator()(OutputListener* ol){
						op(ol, val1, val2);
					}
			};

			class OutputStart{
				public:
					OutputStart(int _n_cols, int _n_rows, std::vector<Chess>& chs)
						:n_cols(_n_cols), n_rows(_n_rows), chesses(chs) { }
				private:
					int n_cols;
					int n_rows;
					std::vector<Chess>& chesses;
				public:
					void operator()(OutputListener* ol){
						ol->start(n_cols, n_rows, chesses);
					}
			};

			struct OpOutputEnd{
				void operator()(OutputListener* ol, OutputListener::EndStatus es){
					ol->end(es);
				}
			};
			typedef OutputBinder1<OutputListener::EndStatus, OpOutputEnd> OutputEnd;

			struct OpOutputSetPlayerName{
				void operator()(OutputListener* ol, Turn turn, Glib::ustring& name){
					ol->setPlayerName(turn, name);
				}
			};
			typedef OutputBinder2<Turn, Glib::ustring&, OpOutputSetPlayerName> OutputSetPlayerName;

			struct OpOutputUpdatePlayer{
				void operator()(OutputListener* ol, Turn turn, int remains){
					ol->updatePlayer(turn, remains);
				}
			};
			typedef OutputBinder2<Turn, int, OpOutputUpdatePlayer> OutputUpdatePlayer;

			struct OpOutputSetCurrentPlayer{
				void operator()(OutputListener* ol, Turn turn){
					ol->setCurrentPlayer(turn);
				}
			};
			typedef OutputBinder1<Turn, OpOutputSetCurrentPlayer> OutputSetCurrentPlayer;

			class OutputMoveChess{
				public:
					OutputMoveChess(int _x1, int _y1, int _x2, int _y2)
						:x1(_x1), y1(_y1), x2(_x2), y2(_y2) { }
				private:
					int x1, y1, x2, y2;
				public:
					void operator()(OutputListener* ol){
						ol->moveChess(x1, y1, x2, y2);
					}
			};

			struct OpOutputUncoverChess{
				void operator()(OutputListener* ol, int x, int y){
					ol->uncoverChess(x, y);
				}
			};
			typedef OutputBinder2<int, int, OpOutputUncoverChess> OutputUncoverChess;

			struct OpOutputRemoveChess{
				void operator()(OutputListener* ol, int x, int y){
					ol->removeChess(x, y);
				}
			};
			typedef OutputBinder2<int, int, OpOutputRemoveChess> OutputRemoveChess;

			class OutputSetSelectedChess{
				public:
					OutputSetSelectedChess(int _x, int _y, bool _select)
						:x(_x), y(_y), select(_select) { }
				private:
					int x;
					int y;
					bool select;
				public:
					void operator()(OutputListener* ol){
						ol->setSelectedChess(x, y, select);
					}
			};

			struct OpOutputSetMessage{
				void operator()(OutputListener* ol, Glib::ustring& message){
					ol->setMessage(message);
				}
			};
			typedef OutputBinder1<Glib::ustring, OpOutputSetMessage> OutputSetMessage;

		protected:
			ChessBoard<8,4>* board;	//FK_,ثeu䴩8x4ѽL
			Player *player1, *player2;
			Player *player_blue, *player_red;
			Turn turn;	//ثea
			Status status;
			std::list<OutputListener*> ols;	//Ӧs<OutputListener&>,æ]YǯSʤŦӵLkϥ
	};
}

#endif
